import { TestBed } from '@angular/core/testing';

import { sampleWithRequiredData, sampleWithNewData } from '../saved-searches.test-samples';

import { SavedSearchesFormService } from './saved-searches-form.service';

describe('SavedSearches Form Service', () => {
  let service: SavedSearchesFormService;

  beforeEach(() => {
    TestBed.configureTestingModule({});
    service = TestBed.inject(SavedSearchesFormService);
  });

  describe('Service methods', () => {
    describe('createSavedSearchesFormGroup', () => {
      it('should create a new form with FormControl', () => {
        const formGroup = service.createSavedSearchesFormGroup();

        expect(formGroup.controls).toEqual(
          expect.objectContaining({
            id: expect.any(Object),
            name: expect.any(Object),
            jsonQuery: expect.any(Object),
            user: expect.any(Object),
          })
        );
      });

      it('passing ISavedSearches should create a new form with FormGroup', () => {
        const formGroup = service.createSavedSearchesFormGroup(sampleWithRequiredData);

        expect(formGroup.controls).toEqual(
          expect.objectContaining({
            id: expect.any(Object),
            name: expect.any(Object),
            jsonQuery: expect.any(Object),
            user: expect.any(Object),
          })
        );
      });
    });

    describe('getSavedSearches', () => {
      it('should return NewSavedSearches for default SavedSearches initial value', () => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const formGroup = service.createSavedSearchesFormGroup(sampleWithNewData);

        const savedSearches = service.getSavedSearches(formGroup) as any;

        expect(savedSearches).toMatchObject(sampleWithNewData);
      });

      it('should return NewSavedSearches for empty SavedSearches initial value', () => {
        const formGroup = service.createSavedSearchesFormGroup();

        const savedSearches = service.getSavedSearches(formGroup) as any;

        expect(savedSearches).toMatchObject({});
      });

      it('should return ISavedSearches', () => {
        const formGroup = service.createSavedSearchesFormGroup(sampleWithRequiredData);

        const savedSearches = service.getSavedSearches(formGroup) as any;

        expect(savedSearches).toMatchObject(sampleWithRequiredData);
      });
    });

    describe('resetForm', () => {
      it('passing ISavedSearches should not enable id FormControl', () => {
        const formGroup = service.createSavedSearchesFormGroup();
        expect(formGroup.controls.id.disabled).toBe(true);

        service.resetForm(formGroup, sampleWithRequiredData);

        expect(formGroup.controls.id.disabled).toBe(true);
      });

      it('passing NewSavedSearches should disable id FormControl', () => {
        const formGroup = service.createSavedSearchesFormGroup(sampleWithRequiredData);
        expect(formGroup.controls.id.disabled).toBe(true);

        service.resetForm(formGroup, { id: null });

        expect(formGroup.controls.id.disabled).toBe(true);
      });
    });
  });
});
